clear all
close all
% Main driver program for basis calculations,
global RBFtype;  % type of RBF see frbf.m
global RBFpar;   % parameter of RBF, see frbf.m
global RBFscale; % scale of RBF, see frbf.m
%
RBFtype='tp';  % basis function tps
RBFscale=1;
RBFpar=2; %
polorder=2
% -----------------------data sites-----------------------------
he=.05;   % h for evaluation grid
[dsites, indd, xee, yee, xe, ye]=cutcircle(he); % generates points
% xe, ye: points of grid covering domain,
% to be used in surf plots
% xee, yee:  points as vectors: xee=xe(:), yee=ye(:)
% dsites: data sites, subset of (xee,yee)
% ind: index set for dsites in [xee,yee]
Np=length(xee(indd)); % number of points for evaluation
% We generate interpolation points here.
% For this test: just take a unisolvent set for polorder+2
indi=getunisolventsubset(dsites, polorder+2);
isites=dsites(indi,:); % the interpolation sites
Ni=length(indi) % number of interpolation points
% now get unisolvent subset for polorder
indu=getunisolventsubset(isites, polorder);
% we need Q
Q=length(indu)
% Now we reorder the isites to have the unisolvent set in front
tempsites=isites(1:Q,:);
isites(1:Q,:)=isites(indu,:);
isites(indu,:)=tempsites;
% plot all of them
figure
plot(dsites(:,1),dsites(:,2),'.',...
    isites(:,1),isites(:,2),'o', isites(1:Q,1),isites(1:Q,2),'rx')
% to avoid confusion, we introduce the unisolvent point set
usites=isites(1:Q,:);
% now evaluate the polynomial Lagrange basis
PUp=polynomials(usites,polorder);
PDu=polynomials(dsites,polorder);
PL=PDu/PUp;
% for i=1:Q
%     figure
%     plotval=zeros(length(xe(:)),1);
%     plotval(indd,1)=PL(:,i);
%     surf(xe, ye, reshape(plotval,size(xe)))
%     title(sprintf('Polynomial Lagrange basis function %d',i));
% end
% disp('Hit Enter to close the plots')
% pause
% close all;
% Now for the full Lagrange CPD RBF basis
Aint=kermat(isites,isites);
Pint=polynomials(isites,polorder);
Afullint=[Aint Pint ; Pint' zeros(Q,Q)];
Aeval=kermat(dsites,isites);
Peval=polynomials(dsites,polorder);
Lrhs=[Aeval Peval];
Lag=Lrhs/Afullint;
% for i=1:Ni
%     figure
%     plotval=zeros(length(xe(:)),1);
%     plotval(indd,1)=Lag(:,i);
%     surf(xe, ye, reshape(plotval,size(xe)))
%     title(sprintf('RBF Lagrange basis function %d',i));
% end
% disp('Hit Enter to close the plots')
% pause
% close all;
% now for the standard basis.
% We need the polynomial Lagrange basis on the full interpolation points
Lagi=PL(indi,:);
SV=Aeval-Aeval(:,1:Q)*(Lagi');
% Note that the first Q columns should be zero:
tSV=norm(SV(:,1:Q))
% for i=Q+1:Ni
%     figure
%     plotval=zeros(length(xe(:)),1);
%     plotval(indd,1)=SV(:,i);
%     surf(xe, ye, reshape(plotval,size(xe)))
%     title(sprintf('Standard basis function %d',i));
% end
% disp('Hit Enter to close the plots')
% pause
% close all;
%

% we now do the projector-based bases
% They just need some C matrix 
PiProj=eye(Ni)-Pint*inv(Pint'*Pint)*Pint';
% now the basis at the end of section 3
C=PiProj;
% B=Pint'; % not needed, see below
% to do SVD or Cholesky, uncomment parts of what follows:
% % we now go for the SVD and Cholesky basis 
% PAP=PiProj'*Aint*PiProj;
% % format long g
% % eig(PAP)
% % here is Cholesky 
% % we cheat a bit on the rank
% L=chol(PAP+eps*eye(Ni),'lower');
% % and fix the cheat in the lower corner
% L(Ni-Q+1:end,Ni-Q+1:end)=eye(Q);
% % to be able to invert:
% Li=inv(L');
% % Now we check if the cheat was OK 
% G=Li'*PAP*Li;
% % the following must be zero if G is near to diagonal
% tng=norm(G-diag(diag(G)))
% % if this was small, the cheat was OK.
% C=PiProj*Li;
% % end of Cholesky
% % % here is SVD 
% % [U,S,V]=svd(PAP);
% % C=PiProj*U;
% % % end of SVD 
% % from here: both SVD and Cholesky and the section 3 stuff
% % we have to add a suitable B matrix 
B=null(C)'; % this is safe 
% B=Pint' % works as well
% and test the rank
tr=rank([C;B])
% The following must be zero in all cases
tnc=norm(Pint'*C)
Vfull=Afullint*[C;B];
% the following must be zero, see the paper
tnv=norm(Vfull(Ni+1:end,:))
% this is the value matrix of the basis
valmat=Vfull(1:Ni,1:Ni);
condv=condest(valmat)
trval=rank(valmat)
% now get the basis on the dsites, see the paper
W=Aeval*C'+Peval*B;
% please check if C' is correct here
for i=1:Ni
    figure
    plotval=zeros(length(xe(:)),1);
    plotval(indd,1)=W(:,i);
    surf(xe, ye, reshape(plotval,size(xe)))
    title(sprintf('Basis function %d',i));
end
disp('Hit Enter to close the plots')
pause
close all;

return


% now for the adaptive Newton basis selecting points of dsites
% We go for Na points in total, and start with Q points.
Na=12
nsites=zeros(Na,2);
nsites(1:Q,:)=usites;
% we follow the paper very closely
% and store the plotvalues of the basis in the array
V=zeros(Np,Na);
% we put the polynomial Lagrange basis into the first Q columns
V(:,1:Q)=PL;
% % We need the final indices of the usites in the dsites:
% inddu=indd(indi(1:Q))
% We do not want to evaluate a kernel on dsites times dsites. NEVER.
% The following vector will be P_Q^2 on the dsites:
KQdd=zeros(Np,1);
for i=1:Np
    KQdd(i,1)=kermat(dsites(i,:),dsites(i,:))-2*PL(i,:)*(Aeval(i,indu))'+...
        PL(i,:)*Aeval(indi(1:Q,1),indu)*PL(i,:)'; 
end
figure
plotval=zeros(length(xe(:)),1);
plotval(indd,1)=KQdd(:,1);
surf(xe, ye, reshape(plotval,size(xe)))
title(sprintf('Power function on %d points',Q));
% it should be zero on the unisolvent set
tnKQdd=norm(KQdd(indi(1:Q),:))
for m=Q:Na-1
    % now get the maximum of the P_m^2 on the dsites
    [vm im]=max(KQdd)
    % This has selected point im
    nsites(m+1,:)=dsites(im,:);
    % Now for the values of K_m(.,x_{m+1})
    Km=kermat(dsites,dsites(im,:));
    KQm=Km-PL*Aeval(im,1:Q)'-Aeval(:,1:Q)*PL(im,:)'+PL*Aint(1:Q,1:Q)*PL(im,:)';
    % should be zero on the Q unisolvent points
    tnKQm=norm(KQm(indi(1:Q),1))
    KQm(indi(1:Q))
    Kval=KQm;
    if m>Q
        Kval=Kval-V(:,Q+1:m)*(V(im,Q+1:m))';
    end
    % now the new Newton basis function 
    V(:,m+1)=Kval/sqrt(Kval(im,1));
    % it should be zero on the first Q points
    tnb=norm(V(indi(1:Q),m+1))
    % We upgrade the power function squared:
    KQdd=KQdd-V(:,m+1).^2;
    % the following should now be zero:
    tmdKQ=abs(KQdd(im,1))
    figure
    plotval=zeros(length(xe(:)),1);
    plotval(indd,1)=V(:,m+1);
    surf(xe, ye, reshape(plotval,size(xe)))
    title(sprintf('Newton basis function %d',m+1));
    figure
    plotval=zeros(length(xe(:)),1);
    plotval(indd,1)=KQdd(:,1);
    surf(xe, ye, reshape(plotval,size(xe)))
    title(sprintf('Power function on %d points',m+1));
    
end
disp('Hit Enter to close the plots')
pause
close all;

figure
plot(dsites(:,1),dsites(:,2),'.',nsites(:,1),nsites(:,2),'ro')
title('Final selected points') 
figure
plotval=zeros(length(xe(:)),1);
plotval(indd,1)=KQdd(:,1);
surf(xe, ye, reshape(plotval,size(xe)))
title(sprintf('Power function on %d points',Na));

disp('Hit Enter to close the plots')
pause
close all;



% now for the Newton basis on the X points
% we follow the paper very closely
% and store the plotvalues of the basis in the array
V=zeros(Np,Ni);
% we put the polynomial Lagrange basis into the first Q columns
V(:,1:Q)=PL;
% We do not want to evaluate a kernel on dsites times dsites.
% but we need it on dsites times isites.
KQdi=Aeval-PL*Aint(1:Q,:)-Aeval(:,1:Q)*Lagi'+PL*Aint(1:Q,1:Q)*Lagi';
% the first Q columns must be zero,
% because we have the reduced kernel here...
tnKQdi=norm(KQdi(:,1:Q))
% the rows in the Q usites must also be zero
tnKQdiu=norm(KQdi(indi(1:Q),:))
% Now we evaluate K_Q(x,x) for x in X.
% this must be zero in the first Q components, and nonnegative.
dKQ=diag(KQdi(indi,:));
tndKQ=norm(dKQ(1:Q,1))
for m=Q:Ni-1
    % now get the maximum of the K_m kernel on the diagonal
    [vm im]=max(dKQ);
    % This has selected point im
    % Now for the values of K_m(.,x_{m+1})
    Kval=KQdi(:,im);
    if m>Q
        Kval=Kval-V(:,Q+1:m)*(V(indi(im),Q+1:m))';
    end
    % now the new Newton basis function 
    V(:,m+1)=Kval/sqrt(Kval(indi(im)));
    figure
    plotval=zeros(length(xe(:)),1);
    plotval(indd,1)=V(:,m+1);
    surf(xe, ye, reshape(plotval,size(xe)))
    title(sprintf('Newton basis function %d',m+1));
    % We upgrade the power function squared:
    dKQ=dKQ-V(indi,m+1).^2
    % the following should now be zero:
    tmdKQ=abs(dKQ(m+1,1))
end
disp('Hit Enter to close the plots')
pause
close all;






