clear all
close all
global text
% Main driver program for basis calculations,
global RBFtype;  % type of RBF see frbf.m
global RBFpar;   % parameter of RBF, see frbf.m
global RBFscale; % scale of RBF, see frbf.m
%
RBFtype='tp';  % basis function tps
RBFscale=1;
RBFpar=2; %
polorder=2
% -----------------------data sites-----------------------------
he=.05;   % h for evaluation grid
[dsites, indd, xee, yee, xe, ye]=cutcircle(he); % generates points
% xe, ye: points of grid covering domain,
% to be used in surf plots
% xee, yee:  points as vectors: xee=xe(:), yee=ye(:)
% dsites: data sites, subset of (xee,yee)
% ind: index set for dsites in [xee,yee]
Np=length(xee(indd)); % number of points for evaluation
% We generate interpolation points here.
% For this test: just take a unisolvent set for polorder+2
indi=getunisolventsubset(dsites, polorder+3);
isites=dsites(indi,:); % the interpolation sites
Ni=length(indi) % number of interpolation points
% now get unisolvent subset for polorder
indu=getunisolventsubset(isites, polorder);
% we need Q
Q=length(indu)
% Now we reorder the isites to have the unisolvent set in front
tempsites=isites(1:Q,:);
isites(1:Q,:)=isites(indu,:);
isites(indu,:)=tempsites;
indt=indi(1:Q,1);
indi(1:Q,1)=indi(indu,1);
indi(indu,1)=indt;
indu=(1:Q)';
% plot all of them
plotpoints(dsites, isites);

% to avoid confusion, we introduce the unisolvent point set
usites=isites(1:Q,:);
% now evaluate the polynomial Lagrange basis
PUp=polynomials(usites,polorder);
PDu=polynomials(dsites,polorder);
PL=PDu/PUp;
text='Polynomial Lagrange Basis Function';
plotallfunctions (PL, 1, Q, xe, ye, indd)


% Now for the full Lagrange CPD RBF basis
Aint=kermat(isites,isites);
Pint=polynomials(isites,polorder);
Afullint=[Aint Pint ; Pint' zeros(Q,Q)];
Aeval=kermat(dsites,isites);
Peval=polynomials(dsites,polorder);
Lrhs=[Aeval Peval];
Lag=Lrhs/Afullint;
text='RBF Lagrange Basis Function';
plotallfunctions (Lag, 1, Ni, xe, ye, indd)


% now for the standard basis.
% We need the polynomial Lagrange basis on the full interpolation points
Lagi=PL(indi,:);
SV=Aeval-Aeval(:,1:Q)*(Lagi');
% Note that the first Q columns should be zero:
tSV=norm(SV(:,1:Q))
% ttsV=sum(abs(SV))
text='Standard Basis Function';
plotallfunctions (SV, Q+1, Ni, xe, ye, indd)

% we now do the projector-based bases
% They just need some C matrix 
PiProj=eye(Ni)-Pint*inv(Pint'*Pint)*Pint';
PAP=PiProj'*Aint*PiProj;
% now the basis at the end of section 3
CProj=PiProj;
% % here is Cholesky 
CChol=docholesky (PAP, PiProj, Ni, Q);
% % here is SVD 
CSVD=doSVD(PAP, PiProj);
% now we get the matrix W ob values of the basis
WProj=getbasisfromprojection(CProj, Ni, Pint, Afullint, Aeval, Peval);
WChol=getbasisfromprojection(CChol, Ni, Pint, Afullint, Aeval, Peval);
WSVD =getbasisfromprojection(CSVD , Ni, Pint, Afullint, Aeval, Peval);

text='Projection Basis Function';
plotallfunctions (WProj, 1, Ni, xe, ye, indd)

text='Cholesky Basis Function';
plotallfunctions (WChol, 1, Ni, xe, ye, indd)

text='SVD Basis Function';

plotallfunctions (WSVD , 1, Ni, xe, ye, indd)


% now for the adaptive Newton basis 
% selecting points of dsites adaptively with a greedy technique
% We go for Na points in total, and start with Q points.
Na=Ni
nsites=zeros(Na,2);
nsites(1:Q,:)=usites;
% we follow the paper very closely
% and store the plotvalues of the basis in the array
V=zeros(Np,Na);
% we put the polynomial Lagrange basis into the first Q columns
V(:,1:Q)=PL;
% % We need the final indices of the usites in the dsites:
% inddu=indd(indi(1:Q))
% We do not want to evaluate a kernel on dsites times dsites. NEVER.
% The following vector will be P_Q^2 on the dsites:
KQdd=zeros(Np,1);
for i=1:Np
    KQdd(i,1)=kermat(dsites(i,:),dsites(i,:))-2*PL(i,:)*(Aeval(i,indu))'+...
        PL(i,:)*Aeval(indi(1:Q,1),indu)*PL(i,:)'; 
end
% the Power function should be zero on the unisolvent set
tnKQdd=norm(KQdd(indi(1:Q),:))
% we store the PF for plotting 
Kqfull=zeros(size(V));
Kqfull(:,Q)=KQdd(:,1);
for m=Q:Na-1
    % now get the maximum of the P_m^2 on the dsites
    [vm im]=max(KQdd)
    % This has selected point im
    nsites(m+1,:)=dsites(im,:);
    % Now for the values of K_m(.,x_{m+1})
    Km=kermat(dsites,dsites(im,:));
    KQm=Km-PL*Aeval(im,1:Q)'-Aeval(:,1:Q)*PL(im,:)'+PL*Aint(1:Q,1:Q)*PL(im,:)';
    % should be zero on the Q unisolvent points
    tnKQm=norm(KQm(indi(1:Q),1))
    % KQm(indi(1:Q))
    Kval=KQm;
    if m>Q
        Kval=Kval-V(:,Q+1:m)*(V(im,Q+1:m))';
    end
    % now the new Newton basis function 
    V(:,m+1)=Kval/sqrt(Kval(im,1));
    % it should be zero on the first Q points
    tnb=norm(V(indi(1:Q),m+1))
    % We upgrade the power function squared:
    KQdd=KQdd-V(:,m+1).^2;
    % the following should now be zero:
    tmdKQ=abs(KQdd(im,1))
    Kqfull(:,m+1)=KQdd(:,1);
    
end
text='Greedy-selected Newton Basis Function';
plotallfunctions (V, Q, Na, xe, ye, indd)
text='Power Function';
plotallfunctions (Kqfull, Q, Na, xe, ye, indd)
plotpoints(dsites, nsites);




% now for the Newton basis on the X points
% we follow the paper very closely
% and store the plotvalues of the basis in the array
V=zeros(Np,Ni);
% we put the polynomial Lagrange basis into the first Q columns
V(:,1:Q)=PL;
% We do not want to evaluate a kernel on dsites times dsites.
% but we need it on dsites times isites.
KQdi=Aeval-PL*Aint(1:Q,:)-Aeval(:,1:Q)*Lagi'+PL*Aint(1:Q,1:Q)*Lagi';
% the first Q columns must be zero,
% because we have the reduced kernel here...
tnKQdi=norm(KQdi(:,1:Q))
% the rows in the Q usites must also be zero
tnKQdiu=norm(KQdi(indi(1:Q),:))
% Now we evaluate K_Q(x,x) for x in X.
% this must be zero in the first Q components, and nonnegative.
dKQ=diag(KQdi(indi,:));
tndKQ=norm(dKQ(1:Q,1))
for m=Q:Ni-1
    % now get the maximum of the K_m kernel on the diagonal
    [vm im]=max(dKQ);
    % This has selected point im
    % Now for the values of K_m(.,x_{m+1})
    Kval=KQdi(:,im);
    if m>Q
        Kval=Kval-V(:,Q+1:m)*(V(indi(im),Q+1:m))';
    end
    % now the new Newton basis function 
    V(:,m+1)=Kval/sqrt(Kval(indi(im)));
    % We upgrade the power function squared:
    dKQ=dKQ-V(indi,m+1).^2;
    % the following should now be zero:
    tmdKQ=abs(dKQ(m+1,1))
end
text='Newton Basis Function';
plotallfunctions (V, Q, Ni, xe, ye, indd)







